"""
AR_Tokens v1.0.2

Author: Arttu Rautio (aturtur)
Website: http://aturtur.com/
Version: 1.0.2

A small collection of custom tokens that might be useful.

Written for Maxon Cinema 4D R21.207
Python version 2.7.14

Original version: 
2021.01.16

Updated:
2021.09.02 - v1.0.2 - Fixed render engine token
                      Added object name token
                      Added project version token

"""
#----------------------------------------------------------------------------------------------------------------------------------------
# Libraries
#----------------------------------------------------------------------------------------------------------------------------------------
import os
import fileinput
import c4d
import maxon
import re
from datetime import datetime
#----------------------------------------------------------------------------------------------------------------------------------------
# Global variables
#----------------------------------------------------------------------------------------------------------------------------------------
delimiter = "_v" # String that indicates version delimiter (used in GetProjectVersion render token)

#----------------------------------------------------------------------------------------------------------------------------------------
# Functions
#----------------------------------------------------------------------------------------------------------------------------------------

# Get version number from file path
def GetVersion(filePath):
    """ Get version number from a file path"""
    versionList = re.findall(delimiter+"\d+",filePath) # Search delimiter+digits (e.g. _v001) string in file path and store those in a [list]
    if len(versionList) == 0: # If no versions found
        return None, None
    rawVersion = re.compile(delimiter).split(versionList[len(versionList)-1])[1] # Version number with zero padding [string]
    version = int(rawVersion) # Version number without zero padding [integer]
    return version, rawVersion # (e.g. 1, 001)

# Date functions
def GetFullYear(data):
    now = datetime.now()
    return now.strftime("%Y")

def GetYear(data):
    now = datetime.now()
    return now.strftime("%y")

def GetMonth(data):
    now = datetime.now()
    return now.strftime("%m")

def GetDay(data):
    now = datetime.now()
    return now.strftime("%d")

# Time
def GetHour(data):
    now = datetime.now()
    return now.strftime("%H")

def GetMinute(data):
    now = datetime.now()
    return now.strftime("%M")

def GetSecond(data):
    now = datetime.now()
    return now.strftime("%S")

# Computer info functions
def GetComputerName(data):
    return os.environ['COMPUTERNAME']

def GetUserName(data):
    return os.environ['USERNAME']

# Render info
def GetRendererName(data):
    renderData = data[1]
    rendererId = renderData[c4d.RDATA_RENDERENGINE]
    if rendererId is not 0:
        return c4d.plugins.FindPlugin(rendererId, 0).GetName().split(' ', 1)[0]
    else:
        return "Standard"

# Object Name
def GetObjectName(data):
    obj = data[0].GetFirstObject()
    if obj == None:
        return ""
    else:
        return obj.GetName()

# Project Version
def GetProjectVersion(data):
    prjName = data[0].GetDocumentName()
    ver = GetVersion(prjName)[1]
    if ver == None:
        return ""
    else:
        return ver

#----------------------------------------------------------------------------------------------------------------------------------------
# Register Tokens
#----------------------------------------------------------------------------------------------------------------------------------------
if __name__=="__main__":
    for registeredToken in c4d.modules.tokensystem.GetAllTokenEntries():
        if registeredToken.get("_token") in ["YYYY", "YY", "MM", "DD", "hh", "mm", "ss", "pcname", "pcuser", "renderer", "objname", "ver"]:
            exit()

    # Time and Date
    c4d.plugins.RegisterToken("YYYY","Date Year Full", "2019", GetFullYear)
    c4d.plugins.RegisterToken("YY",  "Date Year Short", "19",  GetYear)
    c4d.plugins.RegisterToken("MM",  "Date Month",    "00",    GetMonth)
    c4d.plugins.RegisterToken("DD",  "Date Day",      "00",    GetDay)
    c4d.plugins.RegisterToken("hh",  "Date Hour",     "00",    GetHour)
    c4d.plugins.RegisterToken("mm",  "Date Minute",   "00",    GetMinute)
    c4d.plugins.RegisterToken("ss",  "Date Second",   "00",    GetSecond)

    # Computer info
    c4d.plugins.RegisterToken("pcname", "PC Name",   "String", GetComputerName)
    c4d.plugins.RegisterToken("pcuser", "Username",  "String", GetUserName)

    # Render info
    c4d.plugins.RegisterToken("renderer", "Render Engine", "Standard", GetRendererName)

    # Special
    c4d.plugins.RegisterToken("objname", "Object Name", "Cube", GetObjectName)
    c4d.plugins.RegisterToken("ver", "Project Version", "001", GetProjectVersion)